/*
 * 2013,9 Andrew <jing.guo@cs2c.com.cn>
 *
 *
 * Authors:
 * 	Andrew <jing.guo@cs2c.com.cn>
 *
 *
 */
#ifndef __ISIC_H
#define __ISIC_H

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <sys/types.h>
#include <attr/xattr.h>

enum mac_mls_type { MLS_TYPE_UNDEF = 0, MLS_TYPE_LOW , MLS_TYPE_GEN, MLS_TYPE_HIGH, MLS_TYPE_PUBLIC };
enum mac_mls_rank { MLS_RANK_UNDEF = 0, MLS_RANK_MIN = 1, MLS_RANK_MAX=16};
enum mac_iac_lvl {IAC_LVL_UNDEF = 0, IAC_LVL_MIN = 1, IAC_LVL_MAX = 16, IAC_LVL_PUBLIC = 32};
enum isic_log_type {ISIC_LOG_NONE = 0, ISIC_LOG_DENY, ISIC_LOG_ALLOW, ISIC_LOG_ALL};


#define ISIC_ATTR	"security.ISIC"
typedef unsigned char u8;

#define ENOXATTR			999
#define MLS_CAT_SIZE		256
#define NAME_SIZE	4097
#define BUF_SIZE	1024
#define IS_OKSIZE(x) (0<=x&&x<MLS_CAT_SIZE?1:0)

#define FILE_POLICY 	"/etc/isic/file"
#define USER_POLICY 	"/etc/isic/user"

#define isic_log_format(lvl, format, arg...) syslog(lvl, "fun(%s): "format, __FUNCTION__, ##arg)	
#define isic_log(format, arg...)  isic_log_format(LOG_ERR|LOG_USER, format, ##arg )

#define isic_printf(format, arg...) fprintf(stderr,"isic: "format, ##arg)


struct mls_lvl
{
	u8 mls_type; //MLS_TYPE_HIGH/MLS_TYPE_LOW/MLS_TYPE_GEN
	u8 mls_rank; //g0,g1,g2
	u8 catsum;
	u8 mls_cat[MLS_CAT_SIZE];
};

struct isic_lvl 
{
	struct mls_lvl mls_lvl;
	u8 iac_lvl;
};

struct isic_opt
{
	int type;
	const char *value;
};

int convert_lvl_tostring(struct isic_lvl *lvl, char *buf, int len);
int convert_string_tolvl(struct isic_lvl *lvl , char *string, int len);
int convert_string_tomls(struct isic_lvl *lvl , char *string, int len);
int isic_set_task(int pid, struct isic_lvl *lvl);
int isic_get_task(int pid, struct isic_lvl *lvl);
int isic_set_task_byuser(int pid, int uid);
int isic_label_dir(char *dir, struct isic_lvl *lvl);
int isic_label_file(char *file, struct isic_lvl *lvl);
int isic_set_file(char *file, struct isic_lvl *lvl);
int isic_get_file(char *file, struct isic_lvl *lvl);
int isic_add_user(int uid, struct isic_lvl *lvl);
int isic_del_user(int uid);
int isic_set_user(int uid, struct isic_lvl *lvl);
int isic_get_user(int uid, struct isic_lvl *lvl);

static void inline isic_public_lvl(struct isic_lvl *lvl)
{
	if(lvl==NULL)
		return;
	memset(lvl, 0, sizeof(struct isic_lvl));
	lvl->mls_lvl.mls_type = MLS_TYPE_PUBLIC;
	lvl->iac_lvl = IAC_LVL_PUBLIC;
}

#endif
